import datetime
import sys
import shutil
import tempfile

from quixote import cleanup
from wcs import formdef, publisher
from wcs.formdef import FormDef

sys.modules['formdef'] = formdef

def setup_module(module):
    cleanup()

    global pub

    publisher.WcsPublisher.APP_DIR = tempfile.mkdtemp()
    pub = publisher.WcsPublisher.create_publisher()
    pub.cfg['language'] = {'language': 'en'}

def teardown_module(module):
    shutil.rmtree(pub.APP_DIR)


def test_is_disabled():
    formdef = FormDef()
    assert not formdef.is_disabled()

    formdef.disabled = True
    assert formdef.is_disabled()


def test_is_disabled_publication_date():
    formdef = FormDef()

    formdef.publication_date = '%s-%02d-%02d' % (datetime.datetime.today() - datetime.timedelta(1)).timetuple()[:3]
    assert not formdef.is_disabled()

    formdef.publication_date = '%s-%02d-%02d' % (datetime.datetime.today() + datetime.timedelta(1)).timetuple()[:3]
    assert formdef.is_disabled()


def test_is_disabled_expiration_date():
    formdef = FormDef()

    formdef.expiration_date = '%s-%02d-%02d' % (datetime.datetime.today() - datetime.timedelta(1)).timetuple()[:3]
    assert formdef.is_disabled()

    formdef.expiration_date = '%s-%02d-%02d' % (datetime.datetime.today() + datetime.timedelta(1)).timetuple()[:3]
    assert not formdef.is_disabled()
