import sys
import shutil
import StringIO
import tempfile

from quixote import cleanup
from wcs import publisher

from wcs.formdef import FormDef, fields
from wcs.admin.forms import ET, indent


def setup_module(module):
    cleanup()

    global pub, req

    publisher.WcsPublisher.APP_DIR = tempfile.mkdtemp()
    pub = publisher.WcsPublisher.create_publisher()


def teardown_module(module):
    shutil.rmtree(pub.APP_DIR)

def assert_import_export_works(formdef, include_id=False):
    formdef2 = FormDef.import_from_xml_tree(
            formdef.export_to_xml(include_id=include_id), include_id=include_id)
    assert ET.tostring(indent(formdef.export_to_xml(include_id=include_id))
            ) == ET.tostring(indent(formdef2.export_to_xml(include_id=include_id)))
    return formdef2

def test_empty():
    formdef = FormDef()
    formdef.name = 'empty'
    assert_import_export_works(formdef)

def test_text_attributes():
    formdef = FormDef()
    formdef.name = 'Foo'
    formdef.url_name = 'foo'
    f2 = assert_import_export_works(formdef)
    assert f2.url_name == formdef.url_name

def test_boolean_attributes():
    formdef = FormDef()
    formdef.name = 'Foo'
    formdef.url_name = 'foo'
    formdef.confirmation = True
    formdef.allow_drafts = True
    f2 = assert_import_export_works(formdef)
    assert f2.allow_drafts == formdef.allow_drafts
    assert f2.confirmation == formdef.confirmation

def test_a_field():
    formdef = FormDef()
    formdef.name = 'Foo'
    formdef.fields = [
            fields.StringField(type='string', id=1, label='Bar', size=40)
            ]
    f2 = assert_import_export_works(formdef)
    assert len(f2.fields) == len(formdef.fields)

def test_more_fields():
    formdef = FormDef()
    formdef.name = 'Blah'
    formdef.fields = [
            fields.TextField(type='text', label='Bar', pre=True),
            fields.EmailField(type='email', label='Bar'),
            fields.BoolField(type='bool', label='Bar'),
            fields.DateField(type='date', label='Bar', minimum_date='2014-01-01'),
            fields.ItemField(type='item', label='Bar', items=['foo', 'bar', 'baz']),
            ]
    f2 = assert_import_export_works(formdef)
    assert len(f2.fields) == len(formdef.fields)
    assert f2.fields[2].type == formdef.fields[2].type
    assert f2.fields[3].minimum_date == formdef.fields[3].minimum_date
    assert f2.fields[4].items == formdef.fields[4].items

def test_include_id():
    formdef = FormDef()
    formdef.name = 'Blah'
    formdef.fields = [
            fields.TextField(type='text', label='Bar', pre=True),
            fields.EmailField(type='email',  label='Bar'),
            fields.BoolField(type='bool', label='Bar'),
            fields.DateField(type='date', label='Bar', minimum_date='2014-01-01'),
            fields.ItemField(type='item', label='Bar', items=['foo', 'bar', 'baz']),
            ]
    for field in formdef.fields:
        field.id = formdef.get_new_field_id()
    formdef.fields[4].id = '10'
    f2 = assert_import_export_works(formdef, include_id=True)
    assert len(f2.fields) == len(formdef.fields)
    assert f2.fields[0].id == formdef.fields[0].id
    assert f2.fields[4].id == formdef.fields[4].id

