# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import base64
import hmac
import hashlib
import datetime

from quixote import get_request, get_publisher

def get_user_from_api_query_string():
    query_string = get_request().get_query()
    if not query_string:
        return None
    # verify signature
    if not get_request().form.get('orig'):
        return None
    key = get_publisher().get_site_option(get_request().form.get('orig'), 'api-secrets')
    if not key:
        return None
    signature = get_request().form.get('signature')
    if not signature:
        return None
    algo = get_request().form.get('algo')
    try:
        algo = getattr(hashlib, algo)
    except AttributeError:
        algo = hashlib.sha256
    if signature != base64.standard_b64encode(hmac.new(key,
                            query_string[:query_string.find('&signature=')],
                            algo).digest()):
        return None
    delta = (datetime.datetime.utcnow().replace(tzinfo=None) -
             datetime.datetime.strptime(get_request().form.get('timestamp'),
                     '%Y-%m-%dT%H:%M:%SZ'))
    if abs(delta) > datetime.timedelta(seconds=30):
        return None

    user = None
    # XXX: we need to index on email/NameID, to accelerate that part
    if get_request().form.get('email'):
        email = get_request().form.get('email')
        users = list(get_publisher().user_class.select(lambda x: x.email == email))
        if users:
            user = users[0]
    elif get_request().form.get('NameID'):
        ni = get_request().form.get('NameID')
        users = list(get_publisher().user_class.select(lambda x: ni in x.name_identifiers))
        if users:
            user = users[0]

    return user
