# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import tempfile
import os

from M2Crypto import X509, SMIME, BIO

from qommon.storage import StorableObject

class Certificate:

    def __init__(self, cert, type = 'x509'):
        """type : x509 or pkcs7"""
        self.cert = cert
        self.type = type
        self.issuer = ""
        self.subject = ""
        if self.type == "pkcs7":
            self.__init_p7()
        elif self.type == 'x509':
            self.__init_x509()

    def __init_p7(self):
        signers = []
        name = self.__create_tmpfile(self.cert)
        p7 = SMIME.load_pkcs7(name)
        os.remove(name)
        sk = p7.get0_signers(X509.X509_Stack())
        while len(sk):
            x509 = sk.pop()
            self.issuer += x509.get_issuer().as_text()
            self.subject += x509.get_subject().as_text()

    def __init_x509(self):
        x509 = X509.load_cert_string(self.cert)
        self.issuer += x509.get_issuer().as_text()
        self.subject += x509.get_subject().as_text()
        
    def __create_tmpfile(self, data):
        """Return the file name"""
        fd, name = tempfile.mkstemp()
        file = open(name, "w+b")
        file.write(data)
        file.close()
        return name

    def __validate_p7(self, data):
        """data: string
        """
        res = False
        s = SMIME.SMIME()
        # Load the cert
        tmpfile = self.__create_tmpfile(self.cert)
        p7 = SMIME.load_pkcs7(tmpfile)
        os.remove(tmpfile)
    
        text = data
        data_bio = BIO.MemoryBuffer(text)
        sk = p7.get0_signers(X509.X509_Stack())
        s.set_x509_stack(sk)

        # Load CAs cert
        cas = CertificateAuthorities.select(lambda x: x.ca == True)
        st = X509.X509_Store()
        for ca in cas:
            tmpfile = self.__create_tmpfile(ca.cert)
            st.load_info(tmpfile)
            os.remove(tmpfile)
        s.set_x509_store(st)

        try:
            s.verify(p7, data_bio)
            return True
        except SMIME.PKCS7_Error:
            return False

    def validate(self, data):
        """ Return a boolean """
        if self.type == "pkcs7":
            return self.__validate_p7(data)
        return False

class CertificateAuthorities(StorableObject, Certificate):
    _names = 'cas'

    def __init__(self, cert):
        StorableObject.__init__(self)
        Certificate.__init__(self, cert)

