# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from wcs.qommon.humantime import *

from wcs.workflows import Workflow, WorkflowStatusJumpItem, register_item_class

from wcs.formdata import Evolution
from qommon.form import *
from qommon.publisher import get_publisher_class

class TimeoutWorkflowStatusItem(WorkflowStatusJumpItem):
    description = N_('Change Status on Timeout')
    key = 'timeout'
    waitpoint = True

    timeout = None

    _granularity = 20 * 60 # default: 20 minutes, see bottom of file

    def is_available(cls):
        return False
    is_available = classmethod(is_available)

    def render_as_line(self):
        if self.status and self.get_status() and self.timeout:
            return _('Change Status on Timeout to "%(status)s" after %(time)s') % \
                {'status': self.get_status().name, 'time': seconds2humanduration(self.timeout)}
        else:
            return _('Change Status on Timeout (not completed)')

    def add_parameters_widgets(self, form, parameters, prefix='', formdef=None):
        WorkflowStatusJumpItem.add_parameters_widgets(self, form, parameters, prefix, formdef)
        if 'timeout' in parameters:
            _hint = htmltext(_("ex.: 7 days 1 minute<br/> Usable units of time: %(variables)s. " \
                  '<br/><span class="warning">Minimal duration is %(granularity)s</span>')) % {
                        'variables': ','.join(timewords()),
                        'granularity': seconds2humanduration(self._granularity)}
            form.add(StringWidget, '%stimeout' % prefix, title=_('Timeout'),
                        value=seconds2humanduration(self.timeout), hint=_hint)

    def timeout_parse(self, value):
        try:
            return humanduration2seconds(value)
        except ValueError:
            return None

    def get_parameters(self):
        return ('status', 'timeout')

    def must_jump(self, formdata):
        if formdata.evolution:
            last = formdata.evolution[-1].time
        else:
            last = formdata.receipt_time
        if last is None: # shouldn't happen
            return False
        diff = time.time() - time.mktime(last)
        return diff > self.timeout


register_item_class(TimeoutWorkflowStatusItem)
